@@ -1,4 +1,5 @@
 import gzip
+import importlib
 import logging
 import uuid
 import zlib
@@ -17,6 +18,11 @@ class Server(object):
     This class implements a fully compliant Engine.IO web server with support
     for websocket and long-polling transports.
 
+    :param async_mode: The library used for asynchronous operations. Valid
+                       options are "threading", "eventlet" and "gevent". If
+                       this argument is not given, "eventlet" is tried first,
+                       then "gevent", and finally "threading". The websocket
+                       transport is only supported in "eventlet" mode.
     :param ping_timeout: The time in seconds that the client waits for the
                          server to respond before disconnecting.
     :param ping_interval: The interval in seconds at which the client pings
@@ -41,7 +47,7 @@ class Server(object):
     compression_methods = ['gzip', 'deflate']
     event_names = ['connect', 'disconnect', 'message']
 
-    def __init__(self, ping_timeout=60, ping_interval=25,
+    def __init__(self, async_mode=None, ping_timeout=60, ping_interval=25,
                  max_http_buffer_size=100000000, allow_upgrades=True,
                  http_compression=True, compression_threshold=1024,
                  cookie='io', cors_allowed_origins=None,
@@ -67,6 +73,46 @@ class Server(object):
             else:
                 self.logger.setLevel(logging.ERROR)
 
+        threading = None
+        queue = None
+        queue_class = None
+        websocket = None
+        if async_mode is None or async_mode == 'eventlet':
+            try:
+                threading = importlib.import_module('eventlet.green.threading')
+                queue = importlib.import_module('eventlet.queue')
+                queue_class = 'Queue'
+                websocket = importlib.import_module('eventlet.websocket')
+                async_mode = 'eventlet'
+            except ImportError:
+                pass
+        if async_mode is None or async_mode == 'gevent':
+            try:
+                threading = importlib.import_module('gevent.threading')
+                queue = importlib.import_module('gevent.queue')
+                queue_class = 'JoinableQueue'
+                websocket = None
+                async_mode = 'gevent'
+            except ImportError:
+                pass
+        if async_mode is None or async_mode == 'threading':
+            threading = importlib.import_module('threading')
+            try:
+                queue = importlib.import_module('queue')
+            except ImportError:  # pragma: no cover
+                queue = importlib.import_module('Queue')  # pragma: no cover
+            queue_class = 'Queue'
+            websocket = None
+            async_mode = 'threading'
+        if threading is None:
+            raise ValueError('Invalid async_mode specified')
+        self.async_mode = async_mode
+        self.async = {'threading': threading,
+                      'queue': queue,
+                      'queue_class': queue_class,
+                      'websocket': websocket}
+        self.logger.info('Server initialized for %s.', self.async_mode)
+
     def on(self, event, handler=None):
         """Register an event handler.
 
@@ -230,7 +276,8 @@ class Server(object):
 
     def _upgrades(self, sid):
         """Return the list of possible upgrades for a client connection."""
-        if not self.allow_upgrades or self._get_socket(sid).upgraded:
+        if not self.allow_upgrades or self._get_socket(sid).upgraded or \
+                self.async['websocket'] is None:
             return []
         return ['websocket']
 
